@@ -4,7 +4,7 @@
 
    Author: Ilan Schnell
 */
-#define BITARRAY_VERSION  "2.7.1"
+#define BITARRAY_VERSION  "2.7.4"
 
 #ifdef STDC_HEADERS
 #include <stddef.h>
@@ -29,10 +29,17 @@
 #define Py_UNREACHABLE() abort()
 #endif
 
+/* PY_LITTLE_ENDIAN and PY_BIG_ENDIAN (machine byteorder) are available in
+   Python 3.4.10.  Not sure when exactly they were introduced. */
+#ifndef PY_LITTLE_ENDIAN
+#define PY_LITTLE_ENDIAN  (*((uint64_t *) "\xff\0\0\0\0\0\0\0") == 0xff)
+#endif
+
 #if PY_MAJOR_VERSION >= 3
-#define IS_PY3K
+#define IS_PY3K  1
 #define BYTES_SIZE_FMT  "y#"
 #else
+#define IS_PY3K  0
 /* the Py_MIN and Py_MAX macros were introduced in Python 3.3 */
 #define Py_MIN(x, y)  (((x) > (y)) ? (y) : (x))
 #define Py_MAX(x, y)  (((x) > (y)) ? (x) : (y))
@@ -79,6 +86,9 @@ typedef struct {
 #define BITMASK(self, i)  (((char) 1) << ((self)->endian == ENDIAN_LITTLE ? \
                                           ((i) % 8) : (7 - (i) % 8)))
 
+/* buffer as uint64 array */
+#define WBUFF(self)  ((uint64_t *) (self)->ob_item)
+
 /* assert that .nbits is in agreement with .ob_size */
 #define assert_nbits(self)  assert(BYTES((self)->nbits) == Py_SIZE(self))
 
@@ -124,17 +134,41 @@ static const char ones_table[2][8] = {
     {0x00, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfe},  /* big endian */
 };
 
-/* Return last byte in buffer with pad bits zeroed out.  It is an error if
-   the number of bits in the bitarray is a multiple of 8. */
+/* Return last byte in buffer with pad bits zeroed out.
+   If the length of the bitarray is a multiple of 8 (which includes an empty
+   bitarray), 0 is returned. */
 static inline char
-zeroed_last_byte(bitarrayobject *self)
+zlc(bitarrayobject *self)       /* zlc = zeroed last char */
 {
     const int r = self->nbits % 8;     /* index into mask table */
 
-    assert(r > 0);
+    if (r == 0)
+        return 0;
     return self->ob_item[Py_SIZE(self) - 1] & ones_table[IS_BE(self)][r];
 }
 
+/* Return a uint64_t word representing the last (up to 63) remaining bits
+   of the buffer.  All missing bytes (to complete the word) and padbits are
+   treated as zeros.
+   If the length of the bitarray is a multiple of 64 (which includes an empty
+   bitarray), 0 is returned. */
+static inline uint64_t
+zlw(bitarrayobject *self)       /* zlw = zeroed last word */
+{
+    const Py_ssize_t nbits = self->nbits;
+    const Py_ssize_t nw = 8 * (nbits / 64);  /* bytes in complete words */
+    const int nr = (nbits % 64) / 8;         /* complete remaining bytes */
+    uint64_t res = 0;
+
+    assert(nw + nr == nbits / 8 && nw + nr <= Py_SIZE(self));
+    memcpy((char *) &res, self->ob_item + nw, nr);
+    if (nbits % 8)
+        *(((char *) &res) + nr) = zlc(self);
+
+    assert(nbits % 64 || res == 0);
+    return res;
+}
+
 /* Unless buffer is readonly, zero out pad bits.
    Always return the number of pad bits - leave self->nbits unchanged */
 static inline int
@@ -149,15 +183,37 @@ set_padbits(bitarrayobject *self)
     return 8 - r;
 }
 
-static const unsigned char bitcount_lookup[256] = {
-#define B2(n)  n, n + 1, n + 1, n + 2
-#define B4(n)  B2(n), B2(n + 1), B2(n + 1), B2(n + 2)
-#define B6(n)  B4(n), B4(n + 1), B4(n + 1), B4(n + 2)
-    B6(0), B6(1), B6(1), B6(2)
-#undef B2
-#undef B4
-#undef B6
-};
+/* Population count: count the number of 1's in 'x'. */
+static inline int
+popcnt_64(uint64_t x)
+{
+#if (defined(__clang__) || defined(__GNUC__))
+    return __builtin_popcountll(x);
+#else
+    /* https://en.wikipedia.org/wiki/Hamming_weight popcount64c */
+    const uint64_t m1  = 0x5555555555555555;
+    const uint64_t m2  = 0x3333333333333333;
+    const uint64_t m4  = 0x0f0f0f0f0f0f0f0f;
+    const uint64_t h01 = 0x0101010101010101;
+
+    x -= (x >> 1) & m1;
+    x = (x & m2) + ((x >> 2) & m2);
+    x = (x + (x >> 4)) & m4;
+    return (x * h01) >> 56;
+#endif  /* __GNUC__ */
+}
+
+/* population count of `n` words starting from `w` */
+static inline Py_ssize_t
+popcnt_words(uint64_t *w, Py_ssize_t n)
+{
+    Py_ssize_t cnt = 0;
+
+    assert(n >= 0);
+    while (n--)
+        cnt += popcnt_64(*w++);
+    return cnt;
+}
 
 /* adjust index a manner consistent with the handling of normal slices */
 static inline void
