@@ -4,7 +4,6 @@ Tests for bitarray.util module
 from __future__ import absolute_import
 
 import os
-import re
 import sys
 import base64
 import binascii
@@ -17,8 +16,8 @@ from random import choice, randint, random
 from collections import Counter
 
 from bitarray import (bitarray, frozenbitarray, decodetree, bits2bytes,
-                      get_default_endian, _set_default_endian)
-from bitarray.test_bitarray import Util, skipIf
+                      _set_default_endian)
+from bitarray.test_bitarray import Util, skipIf, SYSINFO, DEBUG
 
 from bitarray.util import (
     zeros, urandom, pprint, make_endian, rindex, strip, count_n,
@@ -30,6 +29,11 @@ from bitarray.util import (
     huffman_code, canonical_huffman, canonical_decode,
 )
 
+if DEBUG:
+    from bitarray._util import _sc_rts, _SEGSIZE  # type: ignore
+else:
+    _SEGSIZE = -1
+
 if sys.version_info[0] == 3:
     from io import StringIO
 else:
@@ -442,6 +446,8 @@ class TestsCount_N(unittest.TestCase, Util):
         self.assertRaises(TypeError, count_n, '', 0)
         self.assertRaises(TypeError, count_n, a, 7.0)
         self.assertRaises(ValueError, count_n, a, 0, 2)
+        self.assertRaisesMessage(ValueError, "n = 1 larger than bitarray "
+                                 "size (len(a) = 0)", count_n, a, 1)
 
     def test_simple(self):
         a = bitarray('111110111110111110111110011110111110111110111000')
@@ -453,23 +459,30 @@ class TestsCount_N(unittest.TestCase, Util):
         self.assertEqual(count_n(a, 0, 0), 0)
         self.assertEqual(count_n(a, 2, 0), 12)
         self.assertEqual(count_n(a, 10, 0), 47)
+        # n < 0
         self.assertRaisesMessage(ValueError, "non-negative integer expected",
-                                 count_n, a, -1, 0) # n < 0
-        self.assertRaisesMessage(ValueError, "n larger than bitarray size",
-                                 count_n, a, 49, 0) # n > len(a)
-        self.assertRaisesMessage(ValueError, "n exceeds total count",
-                                 count_n, a, 12, 0) # n > a.count(0)
+                                 count_n, a, -1, 0)
+        # n > len(a)
+        self.assertRaisesMessage(ValueError, "n = 49 larger than bitarray "
+                                 "size (len(a) = 48)", count_n, a, 49, 0)
+        # n > a.count(0)
+        self.assertRaisesMessage(ValueError, "n = 12 exceeds total count "
+                                 "(a.count(0) = 11)",
+                                 count_n, a, 12, 0)
 
         self.assertEqual(count_n(a, 0), 0)
         self.assertEqual(count_n(a, 20), 23)
         self.assertEqual(count_n(a, 20, 1), 23)
         self.assertEqual(count_n(a, 37), 45)
+        # n < 0
         self.assertRaisesMessage(ValueError, "non-negative integer expected",
-                                 count_n, a, -1) # n < 0
-        self.assertRaisesMessage(ValueError, "n larger than bitarray size",
-                                 count_n, a, 49) # n > len(a)
-        self.assertRaisesMessage(ValueError, "n exceeds total count",
-                                 count_n, a, 38) # n > a.count()
+                                 count_n, a, -1)
+        # n > len(a)
+        self.assertRaisesMessage(ValueError, "n = 49 larger than bitarray "
+                                 "size (len(a) = 48)", count_n, a, 49)
+        # n > a.count(1)
+        self.assertRaisesMessage(ValueError, "n = 38 exceeds total count "
+                                 "(a.count(1) = 37)", count_n, a, 38)
 
         for v in 0, 1:
             for n in range(a.count(v) + 1):
@@ -516,6 +529,18 @@ class TestsCount_N(unittest.TestCase, Util):
             self.assertEqual(count_n(a, 1), i + 1)
             self.assertRaises(ValueError, count_n, a, 2)
 
+    def test_last(self):
+        for N in range(1, 1000):
+            a = zeros(N)
+            a[-1] = 1
+            self.assertEqual(a.count(), 1)
+            self.assertEqual(count_n(a, 1), N)
+            if N == 1:
+                msg = "n = 2 larger than bitarray size (len(a) = 1)"
+            else:
+                msg = "n = 2 exceeds total count (a.count(1) = 1)"
+            self.assertRaisesMessage(ValueError, msg, count_n, a, 2)
+
     def test_large(self):
         for _ in range(100):
             N = randint(100000, 250000)
@@ -527,7 +552,9 @@ class TestsCount_N(unittest.TestCase, Util):
             tc = a.count(v)      # total count
             i = count_n(a, tc, v)
             self.check_result(a, tc, i, v)
-            self.assertRaises(ValueError, count_n, a, tc + 1, v)
+            self.assertRaisesMessage(ValueError, "n = %d exceeds total count "
+                                     "(a.count(%d) = %d)" % (tc + 1, v, tc),
+                                     count_n, a, tc + 1, v)
             for _ in range(20):
                 n = randint(0, tc)
                 i = count_n(a, n, v)
@@ -565,7 +592,7 @@ class TestsBitwiseCount(unittest.TestCase, Util):
             self.assertEqual(count_xor(a, ones), 8 - cnt)
             self.assertEqual(count_xor(a, a), 0)
 
-    def test_bit_count1(self):
+    def test_1(self):
         a = bitarray('001111')
         aa = a.copy()
         b = bitarray('010011')
@@ -592,17 +619,18 @@ class TestsBitwiseCount(unittest.TestCase, Util):
                               bitarray('110', 'big'),
                               bitarray('101', 'little'))
 
-    def test_bit_count_frozen(self):
+    def test_frozen(self):
         a = frozenbitarray('001111')
         b = frozenbitarray('010011')
         self.assertEqual(count_and(a, b), 2)
         self.assertEqual(count_or(a, b), 5)
         self.assertEqual(count_xor(a, b), 3)
 
-    def test_bit_count_random(self):
-        for n in list(range(50)) + [randint(1000, 2000)]:
-            a = urandom(n)
-            b = urandom(n)
+    def test_random(self):
+        for _ in range(100):
+            n = randint(0, 1000)
+            a = urandom(n, self.random_endian())
+            b = urandom(n, a.endian())
             self.assertEqual(count_and(a, b), (a & b).count())
             self.assertEqual(count_or(a, b),  (a | b).count())
             self.assertEqual(count_xor(a, b), (a ^ b).count())
@@ -676,7 +704,7 @@ class TestsBitwiseAny(unittest.TestCase, Util):
             self.check(a, b)
 
     def test_one(self):
-        for n in range(1, 200):
+        for n in range(1, 300):
             a = zeros(n)
             b = urandom(n)
             i = randint(0, n - 1)
@@ -783,20 +811,19 @@ class TestsParity(unittest.TestCase, Util):
         a = bitarray()
         self.assertBitEqual(parity(a), 0)
         par = False
-        for _ in range(100):
+        for _ in range(1000):
             self.assertEqual(parity(a), par)
             a.append(1)
             par = not par
 
-    def test_unused_bits(self):
+    def test_pad_ignored(self):
         a = bitarray(1)
         a.setall(1)
         self.assertTrue(parity(a))
 
     def test_frozenbitarray(self):
-        self.assertBitEqual(parity(frozenbitarray()), 0)
-        self.assertBitEqual(parity(frozenbitarray('0010011')), 1)
-        self.assertBitEqual(parity(frozenbitarray('10100110')), 0)
+        for s, p in [('', 0), ('0010011', 1), ('10100110', 0)]:
+            self.assertBitEqual(parity(frozenbitarray(s)), p)
 
     def test_wrong_args(self):
         self.assertRaises(TypeError, parity, '')
@@ -810,7 +837,9 @@ class TestsParity(unittest.TestCase, Util):
 
     def test_random(self):
         for a in self.randombitarrays():
+            b = a.copy()
             self.assertEqual(parity(a), a.count() % 2)
+            self.assertEqual(a, b)
 
 tests.append(TestsParity)
 
@@ -1191,7 +1220,7 @@ class SC_Tests(unittest.TestCase, Util):
                                      bytearray([0x01, 0x10, c]))
 
     def test_decode_header_overflow(self):
-        nbytes = tuple.__itemsize__
+        nbytes = SYSINFO[1]
         self.assertRaisesMessage(
             OverflowError,
             "sizeof(Py_ssize_t) = %d: cannot read 9 bytes" % nbytes,
@@ -1232,7 +1261,7 @@ class SC_Tests(unittest.TestCase, Util):
             ValueError, "decode error (n=3): 32768 >= 32768",
             sc_decode, b"\x02\x00\x80\xc3\x01\x00\x80\x00\0")
 
-        if tuple.__itemsize__ == 4:
+        if SYSINFO[1] == 4:
             msg = "read 4 bytes got negative value: -2147483648"
         else:
             msg = "decode error (n=4): 2147483648 >= 16"
@@ -1240,7 +1269,7 @@ class SC_Tests(unittest.TestCase, Util):
             ValueError, msg,
             sc_decode, b"\x01\x10\xc4\x01\x00\x00\x00\x80\0")
 
-        if tuple.__itemsize__ == 4:
+        if SYSINFO[1] == 4:
             msg = "read 4 bytes got negative value: -1"
         else:
             msg = "decode error (n=4): 4294967295 >= 16"
@@ -1370,7 +1399,7 @@ class SC_Tests(unittest.TestCase, Util):
             self.assertEqual(m, len(b))
             self.round_trip(a)
 
-        a = zeros(1 << 25)
+        a = zeros(1 << 25, 'big')
         a[0] = 1
         self.assertEqual(
             sc_encode(a),
@@ -1398,6 +1427,36 @@ class SC_Tests(unittest.TestCase, Util):
                 a &= urandom(n, endian)
                 self.round_trip(a)
 
+    @skipIf(not DEBUG)
+    def test_rts_empty(self):
+        rts = _sc_rts(bitarray())
+        self.assertEqual(len(rts), 1)
+        self.assertEqual(rts, [0])
+
+    @skipIf(_SEGSIZE != 32)
+    def test_rts_example(self):
+        # see example before sc_calc_rts() in _util.c
+        a = zeros(987)
+        a[:5] = a[512:515] = a[768:772] = 1
+        self.assertEqual(a.count(), 12)
+        rts = _sc_rts(a)
+        self.assertEqual(len(rts), 5)
+        self.assertEqual(rts, [0, 5, 5, 8, 12])
+
+    @skipIf(not DEBUG)
+    def test_rts_random(self):
+        segbits = 8 * _SEGSIZE
+        for n in range(2000):
+            a = urandom(n)
+            rts = _sc_rts(a)
+            self.assertEqual(len(rts), (n + segbits - 1) // segbits + 1)
+            self.assertEqual(rts[0], 0)
+            self.assertEqual(rts[-1], a.count())
+
+            for i in range(len(rts) - 1):
+                seg_pop = a.count(1, segbits * i, segbits * (i + 1))
+                self.assertEqual(rts[i + 1] - rts[i], seg_pop)
+
 tests.append(SC_Tests)
 
 # ---------------------------------------------------------------------------
